import os
import re
import subprocess
import csv
import datetime
import shutil

import yaml

from jobtriggers import config
from jobtriggers import exceptions as exc


def log_job(logname, item, item_type, task, arch):
    now = datetime.datetime.now()

    with open(logname, 'a') as csvfile:
        writer = csv.writer(csvfile, delimiter=';')
        writer.writerow([now, item, item_type, task, arch])


def parse_yaml_from_file(filename):
    with open(filename, 'r') as datafile:
        try:
            return yaml.safe_load(datafile.read())
        except yaml.YAMLError, e:
            raise exc.TriggerError(e)


def should_schedule(formula_file, item_type):
    formula = parse_yaml_from_file(formula_file)
    try:
        run = formula['scheduling']['run']
    except KeyError:
        run = True

    return run and item_type in formula['input']['args']


def get_branch(release):
    matches = re.findall(r'\.(fc\d{1,2})\.?', release)
    if not matches:
        raise exc.TriggerMsgError('Could not parse disttag from %s' % release)

    return matches[-1].replace('c', '')


def get_repo(pkgname):
    return '/'.join([config.distgit_repo_url, config.distgit_namespace, '%s.git' % pkgname])


def get_distgit_tasks(pkgname, item_type, repo, branch='master'):
    tasks_dir = os.path.join(config.distgit_cache_dir, pkgname)

    # for now, just delete the repo and make a new shallow copy
    if os.path.exists(tasks_dir):
        shutil.rmtree(tasks_dir)

    cmd = ['git', 'clone', '--branch', branch, '--depth', '1', repo, tasks_dir]

    try:
        subprocess.check_output(cmd, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        if 'Could not find remote branch' in e.output and branch != 'master':
            return get_distgit_tasks(pkgname, item_type, repo, branch='master')
        raise exc.TriggerError(e.output)

    tasks = []
    for task in os.listdir(tasks_dir):
        if task.startswith('.'):
            continue
        taskdir = os.path.join(tasks_dir, task)
        formula_file = os.path.join(taskdir, 'runtask.yml')
        if os.path.isdir(taskdir) and should_schedule(formula_file, item_type):
            tasks.append(task)

    return tasks
